@@ -7,12 +7,41 @@ use Scalar::Util;
 use Symbol;
 use Carp;
 
-our $VERSION = "0.020001";
+our $VERSION = "0.03";
 
 use base 'Exporter';
 
 our @EXPORT_OK = qw( encode decode );
 
+our $HAS_FMODE;
+BEGIN {
+    $HAS_FMODE = eval { require FileHandle::Fmode } || '';
+}
+
+sub _get_open_mode {
+    my $fh = shift;
+    if (! FileHandle::Fmode::is_FH($fh)) {
+        return undef;
+    }
+
+    if (FileHandle::Fmode::is_RO($fh)) {
+        return '<';
+
+    } elsif (FileHandle::Fmode::is_WO($fh)) {
+        if (FileHandle::Fmode::is_A($fh)) {
+            return '>>';
+        } else {
+            return '>';
+        }
+
+    } elsif (FileHandle::Fmode::is_A($fh)) {
+        return '+>>'
+
+    } else {
+        return '+<'
+    }
+}
+
 sub encode {
     my $value = shift;
     my $path_expr = shift;
@@ -88,6 +117,7 @@ sub encode {
             }
             if (*{$value}{IO}) {
                 if( $tmpvalue{IO} = encode(fileno(*{$value}{IO}), &$_p, $seen) ) {
+                    $tmpvalue{IOmode} = _get_open_mode(*{$value}{IO}) if $HAS_FMODE;
                     $tmpvalue{IOseek} = sysseek($value, 0, 1);
                 }
             }
@@ -235,6 +265,22 @@ sub _create_anon_ref_of_type {
     }
 }
 
+sub _recreate_fh {
+    my($fileno, $mode) = @_;
+
+    my $fh;
+    if ($mode) {
+        open($fh, $mode . '&=', $fileno)
+            || Carp::carp("Couldn't open filehandle for descriptor $fileno with mode $mode: $!");
+
+    } else {
+        open($fh, '>&=', $fileno)
+        || open($fh, '<&=', $fileno)
+        || Carp::carp("Couldn't open filehandle for descriptor $fileno: $!");
+    }
+    return $fh;
+}
+
 sub decode {
     my($input, $recursive_queue, $recurse_fill) = @_;
 
@@ -287,12 +333,10 @@ sub decode {
         $rv = _create_anon_ref_of_type('GLOB', $value->{PACKAGE}, $value->{NAME});
 
         foreach my $type ( keys %$value ) {
-            next if ($type eq 'NAME' or $type eq 'PACKAGE' or $type eq 'IOseek');
+            next if ($type eq 'NAME' or $type eq 'PACKAGE' or $type eq 'IOseek' or $type eq 'IOMode');
             if ($type eq 'IO') {
                 if (my $fileno = $value->{IO}) {
-                    open($rv, '>&=', $fileno)
-                    || open($rv, '<&=', $fileno)
-                    || Carp::carp("Couldn't open filehandle for descriptor $fileno: $!");
+                    $rv = _recreate_fh($fileno, $value->{IOMode});
                 }
             } elsif ($type eq 'CODE') {
                 *{$rv} = \&_dummy_sub unless ($is_real_glob);
@@ -463,7 +507,7 @@ values in the __values slot may also be encoded this way.
 
 Accepts a single value and returns a copy of the data structure originally
 passed to encode().  __refaddr information is discarded and new copies of
-nested data structures is created.  Self-referential is re-linked to the
+nested data structures is created.  Self-referential data is re-linked to the
 appropriate placxe in the new copy.  Blessed references are re-bless into
 the original packages.
 
@@ -471,7 +515,9 @@ Tied variables are re-tied by localizing the appropriate TIE* method to return
 the tied data.  The variable's original data is filled in before calling tie().
 
 The IO slot of typeglobs is recreated by opening the handle with the same
-descriptor number.
+descriptor number.  If the module FileHandle::Fmode is available, it will be
+re-opened in the same mode as the original.  Otherwise, it will first try
+re-opening the file descriptor in read mode, then write mode.
 
 Coderefs cannot be decoded properly.  They are recreated by returning a
 reference to a dummy sub that returns a message explaning the situation.
@@ -480,7 +526,7 @@ reference to a dummy sub that returns a message explaning the situation.
 
 =head1 SEE ALSO
 
-Devel::hdb
+L<JSON>, L<Sereal>, L<Data::Dumper>, L<FileHandle::Fmode>
 
 =head1 AUTHOR
 
